From 101f77e4ed414de465cbc50085ffff475775276c Mon Sep 17 00:00:00 2001
From: Jan Friesse <jfriesse@gmail.com>
Date: Sun, 8 Jul 2012 09:36:20 +0200
Subject: [PATCH] Support for fs hard limit

Support for hard limit is implemented for NFS, but sadly not for other
filesystems. If softlimit is not set (it's 0), hardlimit should be used.
---
 src/plugins/quota/quota-fs.c |   31 +++++++++++++++++++++++++++++++
 1 files changed, 31 insertions(+), 0 deletions(-)

diff --git a/src/plugins/quota/quota-fs.c b/src/plugins/quota/quota-fs.c
index 446dedb..7c4af2b 100644
--- a/src/plugins/quota/quota-fs.c
+++ b/src/plugins/quota/quota-fs.c
@@ -567,9 +567,15 @@ fs_quota_get_linux(struct fs_quota_root *root, bool group, bool bytes,
 			/* values always returned in 512 byte blocks */
 			*value_r = xdqblk.d_bcount * 512;
 			*limit_r = xdqblk.d_blk_softlimit * 512;
+			if (*limit_r == 0) {
+				*limit_r = xdqblk.d_blk_hardlimit * 512;
+			}
 		} else {
 			*value_r = xdqblk.d_icount;
 			*limit_r = xdqblk.d_ino_softlimit;
+			if (*limit_r == 0) {
+				*limit_r = xdqblk.d_ino_hardlimit;
+			}
 		}
 	} else
 #endif
@@ -601,9 +607,15 @@ fs_quota_get_linux(struct fs_quota_root *root, bool group, bool bytes,
 			*value_r = dqblk.dqb_curblocks;
 #endif
 			*limit_r = dqblk.dqb_bsoftlimit * 1024;
+			if (*limit_r == 0) {
+				*limit_r = dqblk.dqb_bhardlimit * 1024;
+			}
 		} else {
 			*value_r = dqblk.dqb_curinodes;
 			*limit_r = dqblk.dqb_isoftlimit;
+			if (*limit_r == 0) {
+				*limit_r = dqblk.dqb_ihardlimit;
+			}
 		}
 	}
 	return 1;
@@ -634,9 +646,15 @@ fs_quota_get_bsdaix(struct fs_quota_root *root, bool group, bool bytes,
 	if (bytes) {
 		*value_r = (uint64_t)dqblk.dqb_curblocks * DEV_BSIZE;
 		*limit_r = (uint64_t)dqblk.dqb_bsoftlimit * DEV_BSIZE;
+		if (*limit_r == 0) {
+			*limit_r = (uint64_t)dqblk.dqb_bhardlimit * DEV_BSIZE;
+		}
 	} else {
 		*value_r = dqblk.dqb_curinodes;
 		*limit_r = dqblk.dqb_isoftlimit;
+		if (*limit_r == 0) {
+			*limit_r = dqblk.dqb_ihardlimit;
+		}
 	}
 	return 1;
 }
@@ -665,9 +683,16 @@ fs_quota_get_hpux(struct fs_quota_root *root, bool bytes,
 			root->mount->block_size;
 		*limit_r = (uint64_t)dqblk.dqb_bsoftlimit *
 			root->mount->block_size;
+		if (*limit_r == 0) {
+			*limit_r = (uint64_t)dqblk.dqb_bhardlimit *
+				root->mount->block_size;
+		}
 	} else {
 		*value_r = dqblk.dqb_curfiles;
 		*limit_r = dqblk.dqb_fsoftlimit;
+		if (*limit_r == 0) {
+			*limit_r = dqblk.dqb_fhardlimit;
+		}
 	}
 	return 1;
 }
@@ -694,9 +719,15 @@ fs_quota_get_solaris(struct fs_quota_root *root, bool bytes,
 	if (bytes) {
 		*value_r = (uint64_t)dqblk.dqb_curblocks * DEV_BSIZE;
 		*limit_r = (uint64_t)dqblk.dqb_bsoftlimit * DEV_BSIZE;
+		if (*limit_r == 0) {
+			*limit_r = (uint64_t)dqblk.dqb_bhardlimit * DEV_BSIZE;
+		}
 	} else {
 		*value_r = dqblk.dqb_curfiles;
 		*limit_r = dqblk.dqb_fsoftlimit;
+		if (*limit_r == 0) {
+			*limit_r = dqblk.dqb_fhardlimit;
+		}
 	}
 	return 1;
 }
-- 
1.7.1

